---------------------------------------------------------------------
-- Creating a valid test scenario
---------------------------------------------------------------------

-- Restore as "test"
-- Create new filegroup for just data
-- Move the clustered index for each table to new filegroup
--	* create primary key as a constraint
--  * use default fillfactor (=100)
-- Create foreign keys
-- Create indexes for joins (manually index each FK column)
-- Shrink primary filegroup to 60MB
-- Backup all 3 databases

-- Restore DBs
-- Check initial sizes
-- Check fragmentation (none!)

---------------------------------------------------------------------
-- Restore for cleanup/move
---------------------------------------------------------------------

USE master
go

DROP DATABASE SalesDBInts
DROP DATABASE SalesDBGUIDs
DROP DATABASE SalesDBSeqGUIDS
go

RESTORE DATABASE [SalesDBInts] 
	FROM  DISK = N'E:\BACKUPS\SalesDBCompressed.BAK' 
	WITH  FILE = 1,  
	   MOVE N'SalesDBData' TO N'E:\Data\SalesDBIntsData.mdf'
	,  MOVE N'SalesDBSalesDataPartition1' TO N'E:\Data\SalesDBPart1.ndf'
	,  MOVE N'SalesDBSalesDataPartition2' TO N'E:\Data\SalesDBPart2.ndf'
	,  MOVE N'SalesDBSalesDataPartition3' TO N'E:\Data\SalesDBPart3.ndf'
	,  MOVE N'SalesDBSalesDataPartition4' TO N'E:\Data\SalesDBPart4.ndf'
	,  MOVE N'SalesDBLog' TO N'E:\Log\SalesDBIntsLog.ldf'
	,  NOUNLOAD,  STATS = 10,  REPLACE
GO

RESTORE DATABASE [SalesDBGUIDs] 
	FROM  DISK = N'E:\BACKUPS\SalesDBGUIDsCompressed.BAK' 
	WITH  FILE = 1,  
	   MOVE N'SalesDBGuidsData' TO N'E:\Data\SalesDBGuidsData.mdf'
	,  MOVE N'SalesDBGuidsLog' TO N'E:\Log\SalesDBGuidsLog.ldf'
	,  NOUNLOAD,  STATS = 10,  REPLACE
GO

RESTORE DATABASE SalesDBSeqGUIDs
	FROM  DISK = N'E:\BACKUPS\SalesDBSeqGUIDsCompressed.BAK' 
	WITH  FILE = 1,  
	   MOVE N'SalesDBSeqGUIDsData' TO N'E:\Data\SalesDBSeqGUIDsData.mdf'
	,  MOVE N'SalesDBSeqGUIDsLog' TO N'E:\Log\SalesDBSeqGUIDsLog.ldf'
	,  NOUNLOAD,  STATS = 10,  REPLACE
GO

ALTER DATABASE [SalesDBInts] 
ADD FILEGROUP RWData
go

USE [master]
GO
ALTER DATABASE [SalesDBInts] 
	ADD FILE ( NAME = N'SalesDBIntsRWData1'
				, FILENAME = N'E:\Data\SalesDBIntsRWData1.ndf' 
				, SIZE = 1536000KB 
				, FILEGROWTH = 153600KB ) 
	TO FILEGROUP [RWData]
GO

ALTER DATABASE [SalesDBInts] 
	ADD FILE ( NAME = N'SalesDBIntsRWData2'
				, FILENAME = N'E:\Data\SalesDBIntsRWData2.ndf' 
				, SIZE = 1536000KB 
				, FILEGROWTH = 153600KB ) 
	TO FILEGROUP [RWData]
GO

ALTER DATABASE [SalesDBGUIDs] 
ADD FILEGROUP RWData
go

USE [master]
GO
ALTER DATABASE [SalesDBGUIDs] 
	ADD FILE ( NAME = N'SalesDBGUIDsRWData1'
				, FILENAME = N'E:\Data\SalesDBGUIDsRWData1.ndf' 
				, SIZE = 1536000KB 
				, FILEGROWTH = 153600KB ) 
	TO FILEGROUP [RWData]
GO

ALTER DATABASE [SalesDBGUIDs] 
	ADD FILE ( NAME = N'SalesDBGUIDsRWData2'
				, FILENAME = N'E:\Data\SalesDBGUIDsRWData2.ndf' 
				, SIZE = 1536000KB 
				, FILEGROWTH = 153600KB ) 
	TO FILEGROUP [RWData]
GO

ALTER DATABASE [SalesDBSeqGUIDs] 
ADD FILEGROUP RWData
go

USE [master]
GO
ALTER DATABASE [SalesDBSeqGUIDs] 
	ADD FILE ( NAME = N'SalesDBSeqGUIDsRWData1'
				, FILENAME = N'E:\Data\SalesDBSeqGUIDsRWData1.ndf' 
				, SIZE = 1536000KB 
				, FILEGROWTH = 153600KB ) 
	TO FILEGROUP [RWData]
GO

ALTER DATABASE [SalesDBSeqGUIDs] 
	ADD FILE ( NAME = N'SalesDBSeqGUIDsRWData2'
				, FILENAME = N'E:\Data\SalesDBSeqGUIDsRWData2.ndf' 
				, SIZE = 1536000KB 
				, FILEGROWTH = 153600KB ) 
	TO FILEGROUP [RWData]
GO

---------------------------------------------------------------------
-- Restore as Baseline
---------------------------------------------------------------------

ALTER DATABASE SalesDBInts SET RECOVERY SIMPLE
go
ALTER DATABASE SalesDBGUIDs SET RECOVERY SIMPLE
go
ALTER DATABASE SalesDBSeqGUIDs SET RECOVERY SIMPLE
go

---------------------------------------------------------------------
-- All keys have been removed
---------------------------------------------------------------------

---------------------
-- For SalesDBInts
---------------------
USE SalesDBInts
go

ALTER TABLE Customers
ADD CONSTRAINT CustomersPK
		PRIMARY KEY CLUSTERED (CustomerID)
ON RWData
go

ALTER TABLE Products
ADD CONSTRAINT ProductsPK
		PRIMARY KEY CLUSTERED (ProductID)
ON RWData
go

ALTER TABLE Employees
ADD CONSTRAINT EmployeesPK
		PRIMARY KEY CLUSTERED (EmployeeID)
ON RWData
go

ALTER TABLE Sales
ADD CONSTRAINT SalesPK
		PRIMARY KEY CLUSTERED (SalesID)
ON RWData
go

ALTER TABLE Sales
ADD CONSTRAINT SalesToCustomersFK
		FOREIGN KEY (CustomerID)
			REFERENCES Customers (CustomerID)
go

CREATE NONCLUSTERED INDEX IX_SalesToCustomersFK 
ON Sales (CustomerID)
ON RWData

ALTER TABLE Sales
ADD CONSTRAINT SalesToProductsFK
		FOREIGN KEY (ProductID)
			REFERENCES Products (ProductID)
go

CREATE NONCLUSTERED INDEX IX_SalesToProductsFK 
ON Sales (ProductID)
ON RWData
go

ALTER TABLE Sales
ADD CONSTRAINT SalesToEmployeesFK
		FOREIGN KEY (SalesPersonID)
			REFERENCES Employees (EmployeeID)
go

CREATE NONCLUSTERED INDEX IX_SalesToEmployeesFK 
ON Employees (EmployeeID)
ON RWData
go

-- Used UI to remove unused files/filegroups
-- Shrank Data portion to 60MB

sp_helpfile
go

CHECKPOINT
go

DBCC SHRINKFILE('SalesDBLog', TRUNCATEONLY)
go

ALTER DATABASE SalesDBInts
	MODIFY FILE 
		( NAME = 'SalesDBLog'
		, SIZE = 1GB)
go

sp_helpfile
go

---------------------
-- For SalesDBGUIDs & SalesDBSeqGUIDs
---------------------
--USE SalesDBGUIDs
USE SalesDBSeqGUIDs
go

ALTER TABLE Customers
ALTER COLUMN CustomerGUID uniqueidentifier NOT NULL
go

ALTER TABLE Products
ALTER COLUMN ProductGUID uniqueidentifier NOT NULL
go

ALTER TABLE Employees
ALTER COLUMN EmployeeGUID uniqueidentifier NOT NULL
go

ALTER TABLE Sales
ALTER COLUMN SalesGUID uniqueidentifier NOT NULL
go

ALTER TABLE Customers
ADD CONSTRAINT CustomersPK
		PRIMARY KEY CLUSTERED (CustomerGUID)
ON RWData
go

ALTER TABLE Products
ADD CONSTRAINT ProductsPK
		PRIMARY KEY CLUSTERED (ProductGUID)
ON RWData
go

ALTER TABLE Employees
ADD CONSTRAINT EmployeesPK
		PRIMARY KEY CLUSTERED (EmployeeGUID)
ON RWData
go

ALTER TABLE Sales
ADD CONSTRAINT SalesPK
		PRIMARY KEY CLUSTERED (SalesGUID)
ON RWData
go

ALTER TABLE Sales
ADD CONSTRAINT SalesToCustomersFK
		FOREIGN KEY (CustomerGUID)
			REFERENCES Customers (CustomerGUID)
go

CREATE NONCLUSTERED INDEX IX_SalesToCustomersFK 
ON Sales (CustomerGUID)
ON RWData
go

ALTER TABLE Sales
ADD CONSTRAINT SalesToProductsFK
		FOREIGN KEY (ProductGUID)
			REFERENCES Products (ProductGUID)
go

CREATE NONCLUSTERED INDEX IX_SalesToProductsFK 
ON Sales (ProductGUID)
ON RWData
go

ALTER TABLE Sales
ADD CONSTRAINT SalesToEmployeesFK
		FOREIGN KEY (SalesPersonGUID)
			REFERENCES Employees (EmployeeGUID)
go

CREATE NONCLUSTERED INDEX IX_SalesToEmployeesFK 
ON Employees (EmployeeGUID)
ON RWData
go

-- Used UI to remove unused files/filegroups
-- Shrank Data portion to 60MB

sp_helpfile
go

CHECKPOINT
go

DBCC SHRINKFILE('SalesDBSeqGuidsLog', TRUNCATEONLY)
go

ALTER DATABASE SalesDBSeqGuids
	MODIFY FILE 
		( NAME = 'SalesDBSeqGuidsLog'
		, SIZE = 1GB)
go

sp_helpfile
go

USE SalesDBInts
go

sp_changedbowner sa
go

USE SalesDBGUIDs
go

sp_changedbowner sa
go

USE SalesDBSeqGUIDs
go

sp_changedbowner sa
go

---------------------------------------------------------------------
-- Backup all 3 databases
---------------------------------------------------------------------

ALTER DATABASE SalesDBInts SET RECOVERY SIMPLE
go

USE SalesDBInts
go

CHECKPOINT
go

ALTER DATABASE SalesDBInts SET RECOVERY FULL
go

BACKUP DATABASE SalesDBInts 
	TO DISK = N'E:\BACKUPS\SalesDBIntsCompressed(6.7M).BAK' 
WITH COMPRESSION
go

ALTER DATABASE SalesDBGUIDs SET RECOVERY SIMPLE
go

USE SalesDBGUIDs
go

CHECKPOINT
go

ALTER DATABASE SalesDBGUIDs SET RECOVERY FULL
go

BACKUP DATABASE SalesDBGUIDs 
	TO DISK = N'E:\BACKUPS\SalesDBGUIDsCompressed(6.7M).BAK' 
WITH COMPRESSION
go

ALTER DATABASE SalesDBSeqGUIDs SET RECOVERY SIMPLE
go

USE SalesDBSeqGUIDs
go

CHECKPOINT
go

ALTER DATABASE SalesDBSeqGUIDs SET RECOVERY FULL
go

BACKUP DATABASE SalesDBSeqGUIDs 
	TO DISK = N'E:\BACKUPS\SalesDBSeqGUIDsCompressed(6.7M).BAK' 
WITH COMPRESSION
go

