USE master
go

DROP DATABASE IdentityScope
go

CREATE DATABASE IdentityScope
go

USE IdentityScope
go

CREATE TABLE DataModificationLog
(
	DataModificationLogID	int			identity,
	LogDate					datetime2	CONSTRAINT DF_DataModificationLog_LogDate
											DEFAULT sysdatetime(),
	[TableName]		sysname,
	RowID			bigint,
	RowModType		varchar(20),
	PrincipalID		int,
	OriginalLogin	sysname
)
go

CREATE TABLE TestIDParent
(
	TestIDParentID	int			identity,
	TestDate		datetime2	CONSTRAINT DF_TestIDParent_TestDate
								DEFAULT getdate()
)
go

CREATE TRIGGER TestIDParent_LogOnInsert
ON TestIDParent
FOR INSERT
AS
INSERT DataModificationLog ([TableName], RowID, RowModType, PrincipalID, OriginalLogin)
	VALUES ('TestIDParent', @@identity, 'Insert', USER_ID(), ORIGINAL_LOGIN())
go

CREATE TRIGGER TestIDParent_LogOnUpdate
ON TestIDParent
FOR UPDATE
AS
INSERT DataModificationLog ([TableName], RowID, RowModType, PrincipalID, OriginalLogin)
	VALUES ('TestIDParent', @@identity, 'Update', USER_ID(), ORIGINAL_LOGIN())
go

CREATE TRIGGER TestIDParent_LogOnDelete
ON TestIDParent
FOR DELETE
AS
INSERT DataModificationLog ([TableName], RowID, RowModType, PrincipalID, OriginalLogin)
	VALUES ('TestIDParent', @@identity, 'Insert', USER_ID(), ORIGINAL_LOGIN())
go

INSERT TestIDParent DEFAULT VALUES
go

SELECT * FROM TestIDParent
SELECT * FROM DataModificationLog
go

-- Everything seems OK!
-- What about the "child" rows?

CREATE TABLE TestIDChild
(
	TestIDChildRowID	int		identity,
	TestIDParentRow		int		not null,
	LineItemNumber		tinyint	not null
)
go

CREATE TRIGGER TestIDChild_LogOnInsert
ON TestIDChild
FOR INSERT
AS
INSERT DataModificationLog ([TableName], RowID, RowModType, PrincipalID, OriginalLogin)
	VALUES ('TestIDChild', @@identity, 'Insert', USER_ID(), ORIGINAL_LOGIN())
go

CREATE TRIGGER TestIDChild_LogOnUpdate
ON TestIDChild
FOR UPDATE
AS
INSERT DataModificationLog ([TableName], RowID, RowModType, PrincipalID, OriginalLogin)
	VALUES ('TestIDChild', @@identity, 'Update', USER_ID(), ORIGINAL_LOGIN())
go

CREATE TRIGGER TestIDChild_LogOnDelete
ON TestIDChild
FOR DELETE
AS
INSERT DataModificationLog ([TableName], RowID, RowModType, PrincipalID, OriginalLogin)
	VALUES ('TestIDChild', @@identity, 'Insert', USER_ID(), ORIGINAL_LOGIN())
go


-- What about parent/child rows:

DECLARE @ParentID	int
INSERT TestIDParent DEFAULT VALUES
SELECT @ParentID = @@IDENTITY
INSERT TestIDChild VALUES (@ParentID, 1)
INSERT TestIDChild VALUES (@ParentID, 1)
INSERT TestIDChild VALUES (@ParentID, 1)
go

SELECT * FROM TestIDParent
SELECT * FROM TestIDChild
SELECT * FROM DataModificationLog
go

-- Everything looks good - seems like we're OK...

DECLARE @ParentID	int
INSERT TestIDParent DEFAULT VALUES
SELECT @ParentID = @@IDENTITY
INSERT TestIDChild VALUES (@ParentID, 1)
INSERT TestIDChild VALUES (@ParentID, 1)
INSERT TestIDChild VALUES (@ParentID, 1)
go

SELECT * FROM TestIDParent
SELECT * FROM TestIDChild

-- Wait - the parent row is wrong?
-- what happened?

SELECT * FROM DataModificationLog
go

-- SQL Server grabbed the "last identity" value...

DECLARE @ParentID	int
INSERT TestIDParent DEFAULT VALUES
SELECT @ParentID = SCOPE_IDENTITY()
INSERT TestIDChild VALUES (@ParentID, 1)
INSERT TestIDChild VALUES (@ParentID, 1)
INSERT TestIDChild VALUES (@ParentID, 1)
go

SELECT * FROM TestIDParent
SELECT * FROM TestIDChild
go

delete TestIDParent
	where TestDate = '2010-04-10 11:51:58.6930000'
select @@IDENTITY, SCOPE_IDENTITY()
go

update TestIDParent
	set TestDate = SYSDATETIME()
	where TestID = 1
select @@IDENTITY, SCOPE_IDENTITY()
go

SELECT * FROM DataModificationLog
go
